/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>

#define PN_MIDI_IN	0
#define PN_MIDI_OUT 1

#define SYSTEM_MSG		        0xF0
#define CONTROL_CHANGE          0xB0

// The core of a MIDI plugin
void Module::OnMidiData( unsigned long p_clock, unsigned long midi_msg, short pin_id)
{
	// sytem messages are for timing info etc
	// they don't have a channel, just pass them right through
	bool is_system_msg = ( midi_msg & SYSTEM_MSG ) == SYSTEM_MSG;

	if( !is_system_msg )
	{
		// The first byte of a midi message contains the status (note on/off etc) and the channel
		int chan = midi_msg & 0x0f;
		int stat = midi_msg & 0xf0;

		// the next two bytes depend on the type of message
		int b2 = ( midi_msg / 0x100 ) & 0xff;
		int b3 = ( midi_msg / 0x10000 ) & 0xff;

		switch( stat ) {
			case CONTROL_CHANGE:
				{
					b3 = 127 - b3;
				}
			// re-assemble the MIDI message into an 32 bit integer
			midi_msg = stat + chan + (b2 << 8) + (b3 << 16);
			break;
		}
	}

	// send the MIDI
	getPin(PN_MIDI_OUT)->TransmitMIDI( p_clock, midi_msg );
}

Module::Module(seaudioMasterCallback seaudioMaster) : SEModule_base(seaudioMaster)
{
	SET_PROCESS_FUNC(Module::sub_process);
}

Module::~Module()
{
	// nothing to see here, move along.
}

// describe the module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	properties->name = "CC Invert";	// this is the name the end-user will see

	// return a unique string.
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug. No two plugs may have the same id.
	properties->id = "Midi CC Inverter by Guido Scognamiglio";

	// Info, may include Author, Web page whatever
	properties->about = "Midi CC Inverter v.1.0 \n by Guido Scognamiglio (c) 2004 \n http://www.soundfonts.it/vst \n guido@gmlab.it";
	return true;
}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )
	{
	case 0:
		properties->name				= "MIDI In";
		properties->direction			= DR_IN;
		properties->datatype			= DT_MIDI2;
		break;
	case 1:
		properties->name				= "MIDI Out";
		properties->direction			= DR_OUT;
		properties->datatype			= DT_MIDI2;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// Nothing to do here in a pure MIDI plugin
	if( !AreEventsPending() )
	{
		CallHost(seaudioMasterSleepMode);
	}
}

void Module::OnPlugStateChange(SEPin *pin)
{
	// Nothing to do here in a pure MIDI plugin
}